@@ -1,5 +1,5 @@
 /*  Lzip - LZMA lossless data compressor
-    Copyright (C) 2008-2014 Antonio Diaz Diaz.
+    Copyright (C) 2008-2016 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,9 +31,6 @@
 #include "decoder.h"
 
 
-const CRC32 crc32;
-
-
 void Pretty_print::operator()( const char * const msg ) const
   {
   if( verbosity >= 0 )
@@ -42,8 +39,8 @@ void Pretty_print::operator()( const char * const msg ) const
       {
       first_post = false;
       std::fprintf( stderr, "  %s: ", name_.c_str() );
-      for( unsigned i = 0; i < longest_name - name_.size(); ++i )
-        std::fprintf( stderr, " " );
+      for( unsigned i = name_.size(); i < longest_name; ++i )
+        std::fputc( ' ', stderr );
       if( !msg ) std::fflush( stderr );
       }
     if( msg ) std::fprintf( stderr, "%s\n", msg );
@@ -110,7 +107,8 @@ void LZ_decoder::flush_data()
     crc32.update_buf( crc_, buffer + stream_pos, size );
     if( outfd >= 0 && writeblock( outfd, buffer + stream_pos, size ) != size )
       throw Error( "Write error" );
-    if( pos >= buffer_size ) { partial_data_pos += pos; pos = 0; }
+    if( pos >= dictionary_size )
+      { partial_data_pos += pos; pos = 0; pos_wrapped = true; }
     stream_pos = pos;
     }
   }
@@ -119,12 +117,12 @@ void LZ_decoder::flush_data()
 bool LZ_decoder::verify_trailer( const Pretty_print & pp ) const
   {
   File_trailer trailer;
-  const int trailer_size = File_trailer::size( member_version );
-  const unsigned long long member_size = rdec.member_position() + trailer_size;
+  int size = rdec.read_data( trailer.data, File_trailer::size );
+  const unsigned long long data_size = data_position();
+  const unsigned long long member_size = rdec.member_position();
   bool error = false;
 
-  int size = rdec.read_data( trailer.data, trailer_size );
-  if( size < trailer_size )
+  if( size < File_trailer::size )
     {
     error = true;
     if( verbosity >= 0 )
@@ -133,34 +131,27 @@ bool LZ_decoder::verify_trailer( const Pretty_print & pp ) const
       std::fprintf( stderr, "Trailer truncated at trailer position %d;"
                             " some checks may fail.\n", size );
       }
-    while( size < trailer_size ) trailer.data[size++] = 0;
+    while( size < File_trailer::size ) trailer.data[size++] = 0;
     }
 
-  if( member_version == 0 ) trailer.member_size( member_size );
-
-  if( !rdec.code_is_zero() )
-    {
-    error = true;
-    pp( "Range decoder final code is not zero." );
-    }
   if( trailer.data_crc() != crc() )
     {
     error = true;
     if( verbosity >= 0 )
       {
       pp();
-      std::fprintf( stderr, "CRC mismatch; trailer says %08X, data CRC is %08X.\n",
+      std::fprintf( stderr, "CRC mismatch; trailer says %08X, data CRC is %08X\n",
                     trailer.data_crc(), crc() );
       }
     }
-  if( trailer.data_size() != data_position() )
+  if( trailer.data_size() != data_size )
     {
     error = true;
     if( verbosity >= 0 )
       {
       pp();
-      std::fprintf( stderr, "Data size mismatch; trailer says %llu, data size is %llu (0x%llX).\n",
-                    trailer.data_size(), data_position(), data_position() );
+      std::fprintf( stderr, "Data size mismatch; trailer says %llu, data size is %llu (0x%llX)\n",
+                    trailer.data_size(), data_size, data_size );
       }
     }
   if( trailer.member_size() != member_size )
@@ -169,18 +160,18 @@ bool LZ_decoder::verify_trailer( const Pretty_print & pp ) const
     if( verbosity >= 0 )
       {
       pp();
-      std::fprintf( stderr, "Member size mismatch; trailer says %llu, member size is %llu (0x%llX).\n",
+      std::fprintf( stderr, "Member size mismatch; trailer says %llu, member size is %llu (0x%llX)\n",
                     trailer.member_size(), member_size, member_size );
       }
     }
-  if( !error && verbosity >= 2 && data_position() > 0 && member_size > 0 )
+  if( !error && verbosity >= 2 && data_size > 0 && member_size > 0 )
     std::fprintf( stderr, "%6.3f:1, %6.3f bits/byte, %5.2f%% saved.  ",
-                  (double)data_position() / member_size,
-                  ( 8.0 * member_size ) / data_position(),
-                  100.0 * ( 1.0 - ( (double)member_size / data_position() ) ) );
+                  (double)data_size / member_size,
+                  ( 8.0 * member_size ) / data_size,
+                  100.0 * ( 1.0 - ( (double)member_size / data_size ) ) );
   if( !error && verbosity >= 4 )
     std::fprintf( stderr, "data CRC %08X, data size %9llu, member size %8llu.  ",
-                  trailer.data_crc(), trailer.data_size(), trailer.member_size() );
+                  crc(), data_size, member_size );
   return !error;
   }
 
@@ -213,7 +204,7 @@ int LZ_decoder::decode_member( const Pretty_print & pp )
     const int pos_state = data_position() & pos_state_mask;
     if( rdec.decode_bit( bm_match[state()][pos_state] ) == 0 )	// 1st bit
       {
-      const uint8_t prev_byte = get_prev_byte();
+      const uint8_t prev_byte = peek_prev();
       if( state.is_char() )
         {
         state.set_char1();
@@ -223,10 +214,10 @@ int LZ_decoder::decode_member( const Pretty_print & pp )
         {
         state.set_char2();
         put_byte( rdec.decode_matched( bm_literal[get_lit_state(prev_byte)],
-                                       get_byte( rep0 ) ) );
+                                       peek( rep0 ) ) );
         }
       }
-    else
+    else					// match or repeated match
       {
       int len;
       if( rdec.decode_bit( bm_rep[state()] ) != 0 )		// 2nd bit
@@ -250,12 +241,12 @@ int LZ_decoder::decode_member( const Pretty_print & pp )
         else
           {
           if( rdec.decode_bit( bm_len[state()][pos_state] ) == 0 ) // 4th bit
-            { state.set_short_rep(); put_byte( get_byte( rep0 ) ); continue; }
+            { state.set_short_rep(); put_byte( peek( rep0 ) ); continue; }
           }
         state.set_rep();
         len = min_match_len + rdec.decode_len( rep_len_model, pos_state );
         }
-      else
+      else					// match
         {
         const unsigned rep0_saved = rep0;
         len = min_match_len + rdec.decode_len( match_len_model, pos_state );
@@ -272,7 +263,7 @@ int LZ_decoder::decode_member( const Pretty_print & pp )
             {
             rep0 += rdec.decode( direct_bits - dis_align_bits ) << dis_align_bits;
             rep0 += rdec.decode_tree_reversed4( bm_align );
-            if( rep0 == 0xFFFFFFFFU )		// Marker found
+            if( rep0 == 0xFFFFFFFFU )		// marker found
               {
               rep0 = rep0_saved;
               rdec.normalize();
@@ -288,7 +279,7 @@ int LZ_decoder::decode_member( const Pretty_print & pp )
               if( verbosity >= 0 )
                 {
                 pp();
-                std::fprintf( stderr, "Unsupported marker code '%d'.\n", len );
+                std::fprintf( stderr, "Unsupported marker code '%d'\n", len );
                 }
               return 4;
               }
@@ -296,7 +287,7 @@ int LZ_decoder::decode_member( const Pretty_print & pp )
           }
         rep3 = rep2; rep2 = rep1; rep1 = rep0_saved;
         state.set_match();
-        if( rep0 >= dictionary_size || rep0 >= data_position() )
+        if( rep0 >= dictionary_size || ( rep0 >= pos && !pos_wrapped ) )
           { flush_data(); return 1; }
         }
       copy_block( rep0, len );
